@@ -33,6 +33,8 @@ IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "i18nUtil.h"
 #define IC_SIZE 64
 
+extern void do_gc(char *);
+
 /* Set IC values */
 static void
 SetCardAttribute(XICAttribute *value_ret, char *p, XICAttr *ic_attr,
@@ -74,6 +76,7 @@ SetCardAttribute(XICAttribute *value_ret, char *p, XICAttr *ic_attr,
     value_ret->type = ic_attr->type;
     value_ret->value_length = value_length;
     value_ret->value = buf;
+    do_gc(buf);
 
     return;
 }
@@ -99,6 +102,7 @@ SetFontAttribute(XICAttribute *value_ret, char *p, XICAttr *ic_attr,
     FrameMgrFree(fm);
     strncpy(buf, base_name, base_length);
     buf[base_length] = (char)0;
+    do_gc(buf);
 
     value_ret->attribute_id = ic_attr->attribute_id;
     value_ret->name = ic_attr->name;
@@ -505,6 +509,7 @@ GetICValue(Xi18n i18n_core, XICAttribute *attr_ret,
 		attr_ret[n].attribute_id = xic_attr[j].attribute_id;
 		attr_ret[n].name_length = xic_attr[j].length;
 		attr_ret[n].name = malloc(xic_attr[j].length + 1);
+		do_gc(attr_ret[n].name);
 		strcpy(attr_ret[n].name, xic_attr[j].name);
 		attr_ret[n].type = xic_attr[j].type;
 		n++;
