@@ -443,6 +443,8 @@ SyncReplyMessageProc(XIMS ims, IMProtocol *call_data, unsigned char *p)
 							  connect_id);
     CARD16 input_method_ID, input_context_ID;
 
+    if (!client) return;
+
     /* create FrameMgr */
     fm = FrameMgrInit(sync_reply_fr, (char *)p,
 		      _Xi18nNeedSwap(i18n_core, connect_id));
@@ -797,6 +799,7 @@ ResetICMessageProc(XIMS ims, IMProtocol *call_data, unsigned char *p)
     Xi18n i18n_core = ims->protocol;
     FrameMgr fm;
     extern XimFrameRec reset_ic_fr[], reset_ic_reply_fr[];
+    extern void (*cb_cleanup)(CARD16 id);
     register int total_size;
     unsigned char *reply = NULL;
     IMResetICStruct *resetic =
@@ -838,13 +841,14 @@ ResetICMessageProc(XIMS ims, IMProtocol *call_data, unsigned char *p)
     FrameMgrPutToken(fm, input_method_ID);
     FrameMgrPutToken(fm, resetic->icid);
     FrameMgrPutToken(fm, resetic->length);
-    FrameMgrPutToken(fm, resetic->commit_string);
+    if (resetic->length) FrameMgrPutToken(fm, resetic->commit_string);
 
     _Xi18nSendMessage(ims, connect_id,
 		      XIM_RESET_IC_REPLY, 0, reply, total_size);
     /* free FrameMgr */
     FrameMgrFree(fm);
-    XFree(resetic->commit_string);
+    if (resetic->commit_string) XFree(resetic->commit_string);
+    if (cb_cleanup) cb_cleanup(resetic->icid);
     XFree(reply);
     return;
 }
@@ -1322,6 +1326,7 @@ ProcessQueue(XIMS ims, CARD16 connect_id)
     Xi18n i18n_core = ims->protocol;
     Xi18nClient *client = (Xi18nClient *)_Xi18nFindClient(i18n_core,
 							  connect_id);
+    if (!client) return;
 
     while (client->sync == False && client->pending != NULL) {
 	XimProtoHdr *hdr = (XimProtoHdr *)client->pending->p;
@@ -1360,7 +1365,7 @@ _Xi18nMessageHandler(XIMS ims, CARD16 connect_id,
     Xi18nClient *client = (Xi18nClient *)_Xi18nFindClient(i18n_core,
 							  connect_id);
 
-    if (hdr == (XimProtoHdr*)NULL) return;
+    if (!client || hdr == (XimProtoHdr*)NULL) return;
 
     memset(&call_data, 0, sizeof(IMProtocol));
 
