@@ -30,11 +30,15 @@ extern int ami_run_mode;
 extern int ami_editing_mode;
 extern int ami_keyboard;
 extern int ami_ekeyboard;
+extern int bell_intensity;
 
 extern gboolean ami_line_wraping_mode;
 extern gboolean ami_use_underline;
 extern gboolean esc_han_toggle;
 extern gboolean unique_han;
 extern gboolean support_status;
-
+extern gboolean wait_wm;
+extern gboolean allow_shuffle;
+extern char *pix_dir_name;
+extern int dock_style;
 #endif
