@@ -1,18 +1,20 @@
-/* this file is a part of Ami software, (C) Hwang chi-deok 1999 */
+/* this file is a part of Ami software, (C) Hwang chi-deok 1999, 2000, 2001 */
 
 #include "config.h"
 #include "cp.h"
 #include "ami.h"
-#include "hangulinput.h"
+#include "hinput.h"
 #include "conf.h"
 #include "keyparse.h"
+#include "applet.h"
+#include <dirent.h>
 
 static GList *activate_list = NULL;
 
 static void about_ami(void);
 static void warn_no_effect(char *message);
 static gint cp_delete( GtkWidget *widget, void *data );
-static GtkTooltips *tooltip;
+static GtkTooltips *tooltip = NULL;
 static guint update_id = 0;
 
 static void
@@ -56,6 +58,12 @@ esc_mode_toggle_cb(GtkWidget *w, gpointer data)
     esc_han_toggle = GTK_TOGGLE_BUTTON(w)->active;
 }
 
+static void
+shuffle_mode_toggle_cb(GtkWidget *w, gpointer data)
+{
+    allow_shuffle = GTK_TOGGLE_BUTTON(w)->active;
+}
+
 static void
 unique_han_toggle_cb(GtkWidget *w, gpointer data)
 {
@@ -116,6 +124,12 @@ underline_mode_toggled(GtkWidget *w)
     ami_use_underline = GTK_TOGGLE_BUTTON(w)->active;
 }
 
+static void
+bell_changed(GtkAdjustment *adj)
+{
+    bell_intensity = adj->value;
+}
+
 static void
 save_conf(GtkWidget *w, gpointer data)
 {
@@ -130,20 +144,7 @@ save_conf(GtkWidget *w, gpointer data)
 void
 set_keyboard(void)
 {
-    char *s;
-    if (ami_keyboard < 2 || ami_keyboard > 4) {
-	if ((s = g_getenv("AMI_KEYBOARD")) == NULL) 
-	    s = g_getenv("HANGUL_KEYBOARD_TYPE");
-	if (s) {
-	    if (*s == '2') ami_hangul_keyboard_set(2);
-	    else if (!strncmp(s, "3FINAL", 6)) ami_hangul_keyboard_set(4);
-	    else ami_hangul_keyboard_set(3);
-	} else {
-	    ami_hangul_keyboard_set(2);
-	}
-    } else {
-	ami_hangul_keyboard_set(ami_keyboard);
-    }
+    ami_hangul_keyboard_set(ami_keyboard);
 }
 
 static GtkWidget *
@@ -152,7 +153,7 @@ create_keyboard_setting(void)
     /* Ű  */
     GtkWidget *vbox0, *vbox1, *frame, *hbox;
     GtkWidget *qwerty, *dvorak;
-    GtkWidget *button1, *button2, *button3;
+    GtkWidget *button0, *button1, *button2, *button3;
     GSList *group;
     int keyboard;
 
@@ -164,7 +165,11 @@ create_keyboard_setting(void)
 
     vbox1 = gtk_vbox_new(TRUE, 2);
     gtk_container_add(GTK_CONTAINER(frame), vbox1);
-    button1 = gtk_radio_button_new_with_label(NULL, "ι");
+    button0 = gtk_radio_button_new_with_label(NULL, "ȯ溯");
+    group = gtk_radio_button_group(GTK_RADIO_BUTTON(button0));
+    gtk_container_add(GTK_CONTAINER(vbox1), button0);
+
+    button1 = gtk_radio_button_new_with_label(group, "ι");
     group = gtk_radio_button_group(GTK_RADIO_BUTTON(button1));
     gtk_container_add(GTK_CONTAINER(vbox1), button1);
 
@@ -175,18 +180,22 @@ create_keyboard_setting(void)
     button3 = gtk_radio_button_new_with_label(group, " ");
     gtk_container_add(GTK_CONTAINER(vbox1), button3);
 
-    keyboard = ami_hangul_keyboard_get();
-    switch(keyboard) {
+/*    keyboard = ami_hangul_keyboard_get(); */
+    switch(ami_keyboard) {
 	case 2:
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button1), TRUE);
 	    break;
 	case 3:
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button2), TRUE);
 	    break;
-	default:
+	case 4:
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button3), TRUE);
 	    break;
+	default:
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button0), TRUE);
+	    break;
     }
+    gtk_signal_connect(GTK_OBJECT(button0), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(1));
     gtk_signal_connect(GTK_OBJECT(button1), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(2));
     gtk_signal_connect(GTK_OBJECT(button2), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(3));
     gtk_signal_connect(GTK_OBJECT(button3), "toggled", (GtkSignalFunc)keyboard_toggle_cb, GINT_TO_POINTER(4));
@@ -284,14 +293,95 @@ create_input_setting(void)
     gtk_signal_connect(GTK_OBJECT(esc), "toggled", (GtkSignalFunc)unique_han_toggle_cb, NULL);
     gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), esc, "ѱ   ƹ̸ ٽ ž߸ ȿ ֽϴ.", NULL);
 
+    esc = gtk_check_button_new_with_label("/߼ ڹٲ ");
+    gtk_box_pack_start(GTK_BOX(vbox0), esc, FALSE, FALSE, 5);
+    if (allow_shuffle) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(esc), TRUE);
+    }
+    gtk_signal_connect(GTK_OBJECT(esc), "toggled", (GtkSignalFunc)shuffle_mode_toggle_cb, NULL);
+
     return vbox0;
 }
 
+static void
+pix_changed(GtkWidget *w)
+{
+    char * name = gtk_object_get_data(GTK_OBJECT(w), "directory");
+    applet_change_pixmap_dir(name);
+}
+
+static gboolean
+is_valid_pixdir(char *dir)
+{
+    char *s;
+    s = g_strconcat(dir, "/ami-h.xpm", NULL);
+    if (access(s, R_OK) != 0) goto error;
+    g_free(s);
+    s = g_strconcat(dir, "/ami-e.xpm", NULL);
+    if (access(s, R_OK) != 0) goto error;
+    g_free(s);
+    s = g_strconcat(dir, "/ami-d.xpm", NULL);
+    if (access(s, R_OK) != 0) goto error;
+    g_free(s);
+    return TRUE;
+    error:
+    g_free(s);
+    return FALSE;
+}
+
+static GList *
+get_pixmap_dir_list_sub(char *parent_dir, GList *list)
+{
+    DIR *pix_dir;
+    struct dirent *entry;
+    pix_dir = opendir(parent_dir);
+    if (!pix_dir) return list;
+    while((entry=readdir(pix_dir))) {
+	char *s;
+        if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
+	    continue;
+        s = g_strconcat(parent_dir, "/", entry->d_name, NULL);
+        if (!is_valid_pixdir(s)) {
+	    g_free(s);
+	    continue;
+        }
+        list = g_list_prepend(list, s);
+    } 
+    closedir(pix_dir);
+    return list;
+}
+
+static char **
+get_pixmap_dir_list(void)
+{
+    char * pix_dir_name = g_strconcat(g_get_home_dir(), "/.ami", NULL);
+    GList *list = NULL;
+    char **dir_list;
+    int i, n;
+
+    list = get_pixmap_dir_list_sub(pix_dir_name, list);
+    g_free(pix_dir_name);
+    pix_dir_name = g_strconcat(XPM_DIR, "/pix", NULL);
+    list = get_pixmap_dir_list_sub(pix_dir_name, list);
+    g_free(pix_dir_name);
+    if (!list) return NULL;
+    n = g_list_length(list);
+    dir_list = g_new(char *, n+1);
+    for(i=0;i<n;i++) dir_list[i] = g_list_nth(list, i)->data;
+    dir_list[n] = NULL;
+    g_list_free(list);
+    return dir_list;
+}
+
+
 static GtkWidget *
 create_appearance_setting(void)
 {
     GtkWidget *vbox, *frame, *hbox, *status;
     GtkWidget *l, *entry, *under;
+    GtkWidget *bell_box, *bell_label, *bell_scale;
+    GtkObject *adj;
+    char **pixmap_dirs;
 
     vbox = gtk_vbox_new(FALSE, 2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
@@ -343,6 +433,61 @@ create_appearance_setting(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(under), TRUE);
     }
     gtk_signal_connect(GTK_OBJECT(under), "toggled", (GtkSignalFunc)underline_mode_toggled, NULL);
+    bell_box = gtk_hbox_new(FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(vbox), bell_box, FALSE, FALSE, 5);
+    bell_label = gtk_label_new("Ҹ ũ");
+    adj = gtk_adjustment_new (0.0, -100, 110, 1, 10, 10);
+    gtk_signal_connect(adj, "value-changed", (GtkSignalFunc)bell_changed, NULL);
+    bell_scale = gtk_hscale_new((GtkAdjustment*)adj);
+    gtk_range_set_update_policy (GTK_RANGE (bell_scale), GTK_UPDATE_DELAYED);
+    gtk_scale_set_digits (GTK_SCALE (bell_scale), 0);
+    gtk_scale_set_draw_value (GTK_SCALE (bell_scale), TRUE);
+    gtk_box_pack_start(GTK_BOX(bell_box), bell_label, FALSE, FALSE, FALSE);
+    gtk_container_add(GTK_CONTAINER(bell_box), bell_scale);
+
+    if ( ami_run_mode != AMI_NORMAL_RUN && (pixmap_dirs=get_pixmap_dir_list())) {
+	GtkWidget *omenu, *menu, *menu_item;
+	GtkWidget *label, *hbox;
+	GSList *group;
+	int selected, i;
+	if (!pix_dir_name) pix_dir_name = g_strdup("");
+	selected = -1;
+	i = 0;
+	while (pixmap_dirs[i]) {
+	    if (strcmp(pixmap_dirs[i], pix_dir_name) == 0) {
+		selected = i;
+		break;
+	    }
+	    i++;
+	}
+	omenu = gtk_option_menu_new();
+	menu = gtk_menu_new();
+	group = NULL;
+	menu_item = gtk_radio_menu_item_new_with_label(group, "⺻");
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", (GtkSignalFunc)pix_changed, NULL);
+	gtk_menu_append(GTK_MENU(menu), menu_item);
+	if (selected ==  -1)
+	  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	gtk_widget_show(menu_item);
+	i = 0;
+	while(pixmap_dirs[i]) {
+	    menu_item = gtk_radio_menu_item_new_with_label(group, pixmap_dirs[i]);
+	    gtk_signal_connect(GTK_OBJECT(menu_item), "activate", (GtkSignalFunc)pix_changed, NULL);
+	    gtk_menu_append(GTK_MENU(menu), menu_item);
+	    if (selected ==  i)
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	    gtk_widget_show(menu_item);
+	    gtk_object_set_data(GTK_OBJECT(menu_item), "directory", pixmap_dirs[i]);
+	    i++;
+	}
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), selected + 1);
+	hbox = gtk_hbox_new(FALSE, 2);
+	label = gtk_label_new("׸:");
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), omenu, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+    }
 
     return vbox;
 }
@@ -522,7 +667,7 @@ about_ami(void)
     str = g_string_new(NULL);
     g_string_sprintf(str, "ƹ  %s\n", VERSION);
     g_string_append(str, 
-	"\nƹ̴ Ȳġ(cdhwang@sr.hei.co.kr) \n"
+	"\nƹ̴ Ȳġ(hwang@mizi.co.kr) \n"
 	"gtk+ ̿Ͽ  \nGnu General Public License Version 2"
 	" \n Ӱ Ͻ  ִ ѱԷ±Դϴ.\n\n"
         "gtk+  Ϳ    \n Ѵ.\n"
@@ -544,6 +689,8 @@ void
 cp_quit(void)
 {
     IMCloseIM(xims);
+    xims = NULL;
+    gdk_flush(); /* ??? */
     gtk_main_quit();
 }
 
@@ -551,4 +698,4 @@ static gint cp_delete( GtkWidget *widget, void *data )
 {
     gtk_widget_hide(cp_win); /* hide cp_win */
     return TRUE;
-}
\ No newline at end of file
+}
