@@ -45,8 +45,7 @@ dw_new(IC *ic)
     dw->text = g_strdup("");
     dw->area.x = -1;
     dw->ic = ic;
-    dw->win = NULL;
-    if (unique_han && ic->top_win == NULL) ic->top_win = GDK_ROOT_PARENT();
+    if ((unique_han || ic->input_style == (XIMStatusNone|XIMPreeditNone))&& ic->top_win == NULL) ic->top_win = GDK_ROOT_PARENT();
     return dw;
 }
 
@@ -54,16 +53,25 @@ int
 dw_status_timeout_show(gpointer data)
 {
     IC *ic;
+    IC *status_ic;
     CARD16 id;
     id = GPOINTER_TO_INT(data);
     ic = ic_find_with_id(id);
     if (!ic) return 0;
     if (!ic->status_win) return 0;
-    dw_show(ic->status_win);
+    status_ic = cli_get_status_ic(ic);
+    if (status_ic == NULL || status_ic == ic) dw_show(ic->status_win);
 
     return 0;
 }
 
+static void
+report_parent(GdkWindow *win)
+{
+    gint x, y, width, height, depth;
+    gdk_window_get_geometry (win,&x,&y,&width,&height,&depth);
+}
+
 void
 dw_set_parent(DispWindow *dw, GdkWindow *parent)
 {
@@ -72,11 +80,18 @@ dw_set_parent(DispWindow *dw, GdkWindow *parent)
     g_return_if_fail (dw != NULL);
     g_return_if_fail (parent != NULL);
 
+#if 0
+    g_print("ic=%d parent info: ", dw->ic->id);
+    report_parent(parent);
+#endif
     if ((dw->ic->input_style & XIMStatusArea) && dw->ic->top_win == NULL) {
 	dw->ic->top_win = dw->ic->client_win;
     }
 
     if (dw->win) {
+	XWindowAttributes xwa;
+	XSetWindowAttributes set_xwa;
+	Window xwin;
 	if (gdk_window_get_parent(dw->win) == parent) {
 	    return;
 	}
@@ -93,6 +108,8 @@ dw_set_parent(DispWindow *dw, GdkWindow *parent)
     attr.event_mask = GDK_EXPOSURE_MASK;
     attr.wclass = GDK_INPUT_OUTPUT;
     attr.window_type = GDK_WINDOW_CHILD;
+    attr.width = 1;
+    attr.height = 1;
     if (dw->caret >= 0) {
 	if (dw->x < dw->area.x) dw->x = dw->area.x;
 	if (dw->y - dw->font->ascent < dw->area.y) dw->y = dw->area.y + dw->font->ascent;
@@ -107,12 +124,12 @@ dw_set_parent(DispWindow *dw, GdkWindow *parent)
     gdk_window_add_filter (dw->win, (GdkFilterFunc)dw_event_filter, dw);
 
     dw->gc = gdk_gc_new(dw->win);
-    dw->reverse_gc = gdk_gc_new(dw->win);
+    if (dw->caret >= 0) dw->reverse_gc = gdk_gc_new(dw->win);
 
     gdk_gc_set_stipple(dw->gc, dim_mask);
-    gdk_gc_set_stipple(dw->reverse_gc, dim_mask);
+    if(dw->reverse_gc) gdk_gc_set_stipple(dw->reverse_gc, dim_mask);
     gdk_gc_set_fill(dw->gc, GDK_STIPPLED);
-    gdk_gc_set_fill(dw->reverse_gc, GDK_STIPPLED);
+    if(dw->reverse_gc) gdk_gc_set_fill(dw->reverse_gc, GDK_STIPPLED);
 
     dw_set_fg(dw, dw->fg);
     dw_set_bg(dw, dw->bg);
@@ -284,6 +301,7 @@ void
 dw_set_area(DispWindow *dw, XRectangle *area)
 {
     g_return_if_fail (dw != NULL);
+    if (debug) g_print("%s: ic=%d x=%d, y=%d, w=%d, h=%d\n",__FUNCTION__,dw->ic->id, area->x,area->y,area->width,area->height);
     dw->area = *area;
     if (dw->win == NULL) return;
     dw_update_width(dw);
@@ -294,7 +312,7 @@ dw_set_area(DispWindow *dw, XRectangle *area)
 	    //gdk_window_move(dw->win, dw->x, dw->y - dw->font->ascent);
 	}
     } else {
-	gdk_window_move_resize(dw->win, area->x, area->y, area->width, area->height);
+	//gdk_window_move_resize(dw->win, area->x, area->y, area->width, area->height);
     }
 }
 
@@ -344,7 +362,8 @@ dw_set_text(DispWindow *dw, char *text, int caret, int has_temp_hangul)
 }
 
 static int 
-dont_move_to_next_line (DispWindow *dw) {
+dont_move_to_next_line (DispWindow *dw) 
+{
     int y;
 
     if (ami_line_wraping_mode) {
@@ -366,6 +385,7 @@ dw_set_pos(DispWindow *dw, XPoint *p)
     if (dw->x == p->x && dw->y == p->y) return;
     dw->x = p->x;
     dw->y = p->y;
+    if(debug) g_print("%s: ic=%d x=%d y=%d\n",__FUNCTION__,dw->ic->id, p->x,p->y);
     if (!dw->win) return;
     if (dw->x + string_width(dw, "")> dw->area.width) {
 	if (!dont_move_to_next_line(dw)) {
@@ -380,7 +400,6 @@ dw_set_pos(DispWindow *dw, XPoint *p)
 	}
     }
     gdk_window_move(dw->win, dw->x, dw->y - dw->font->ascent);
-    if (debug) g_print("%s: %d %d\n", __FUNCTION__, dw->x, dw->y);
     dw_update_width(dw);
 }
 
@@ -424,15 +443,24 @@ dw_destroy(DispWindow *dw)
 void
 dw_set_font(DispWindow *dw, char *font)
 {
+    XPoint p;
     g_return_if_fail (dw != NULL);
     g_return_if_fail (font != NULL);
+
+    /* qt2.?.x Ŀ   ۲  . Silly! */
+    if (strcmp(dw->fontname, font) == 0) return;
+
     util_font_unref(dw->font);
     dw->font = util_fontset_load(font);
     g_free(dw->fontname);
     dw->fontname = g_strdup(font);
+    if(debug) g_print("%s: fontname = %s ic=%d\n", __FUNCTION__, font, dw->ic->id);
     /* ũ⸦ 츮 ߴٸ ο ۲ÿ ٽ Ѵ. */
     if (dw->area.x < 0) dw->area.width = 0;
-    dw_update_width(dw);
+    p.x = dw->x;
+    p.y = dw->y;
+    dw->x = -1;
+    dw_set_pos(dw, &p);
 }
 
 char *
@@ -451,7 +479,7 @@ dw_real_set_bg(DispWindow *dw)
     if (ami_use_underline || dw->caret < 0) {
 	bg = dw->bg;
     } else {
-	/* fg bg 1:9    pixel Ѵ. */
+	/* fg bg 2:8    pixel Ѵ. */
 	XColor cfg;
 	XColor cbg;
 	GdkColor newbg;
@@ -468,7 +496,7 @@ dw_real_set_bg(DispWindow *dw)
 	bg = newbg.pixel;
     }
     XSetWindowBackground(gdk_display, GDK_WINDOW_XWINDOW(dw->win), bg);
-    XSetForeground(gdk_display, GDK_GC_XGC(dw->reverse_gc), dw->bg);
+    if (dw->reverse_gc) XSetForeground(gdk_display, GDK_GC_XGC(dw->reverse_gc), dw->bg);
     if (dw->win2) {
     	XSetWindowBackground(gdk_display, GDK_WINDOW_XWINDOW(dw->win2), bg);
     }
@@ -494,7 +522,7 @@ dw_set_fg(DispWindow *dw, CARD32 fg)
     dw->fg = fg;
     if (dw->gc) {
         XSetForeground(gdk_display, GDK_GC_XGC(dw->gc), fg);
-	XSetBackground(gdk_display, GDK_GC_XGC(dw->reverse_gc), fg);
+	if (dw->reverse_gc) XSetBackground(gdk_display, GDK_GC_XGC(dw->reverse_gc), fg);
     }
     if (!ami_use_underline) {
 	dw_real_set_bg(dw);
@@ -532,6 +560,10 @@ dw_show(DispWindow *dw)
 	gdk_window_show(dw->win);
 	if (dw->win2) gdk_window_show(dw->win2);
 	if (dw->win3) gdk_window_show(dw->win3);
+    } else {
+        gdk_window_raise(dw->win);
+	if (dw->win2) gdk_window_raise(dw->win2);
+	if (dw->win3) gdk_window_raise(dw->win3);
     }
 }
 
@@ -699,7 +731,6 @@ dw_draw(DispWindow *dw)
     if (dw->win == NULL) return;
     if (!gdk_window_is_visible(dw->win)) return;
 
-
     if (dw->caret < 0 || dw->ic->input_style & XIMPreeditArea) 
     	dw_draw_simple(dw);
     else if (dont_move_to_next_line(dw))
@@ -733,13 +764,10 @@ dw_update_width(DispWindow *dw)
     }
 
     if (dw->caret < 0) {
-	if (dw->win_width != dw->area.width) {
-	    int x;
-	    if (dw->area.x < 0) x = 0;
-	    else x = dw->area.x;
-	    gdk_window_move_resize(dw->win, x, dw->area.height + dw->area.y - font_height, dw->area.width, font_height);
-	    dw->win_width = dw->area.width;
-	}
+	int x;
+	if (dw->area.x < 0) x = 0;
+	else x = dw->area.x;
+	gdk_window_move_resize(dw->win, x, dw->area.height + dw->area.y - font_height, dw->area.width, font_height);
 	dw_draw(dw);
 	return;
     }
@@ -888,7 +916,7 @@ dw_unset_focus(DispWindow *dw)
     g_return_if_fail(dw != NULL);
     if (dw->gc == NULL) return;
     gdk_gc_set_fill(dw->gc, GDK_STIPPLED);
-    gdk_gc_set_fill(dw->reverse_gc, GDK_STIPPLED);
+    if (dw->reverse_gc) gdk_gc_set_fill(dw->reverse_gc, GDK_STIPPLED);
     dw_draw(dw);
 }
 
@@ -896,9 +924,12 @@ void
 dw_set_focus(DispWindow *dw)
 {
     g_return_if_fail(dw != NULL);
+    if (debug) g_print("%s: ic=%d\n", __FUNCTION__, dw->ic->id);
     if (dw->gc == NULL) return;
+    if (debug) g_print("2 %s: ic=%d\n", __FUNCTION__, dw->ic->id);
     gdk_gc_set_fill(dw->gc, GDK_SOLID);
-    gdk_gc_set_fill(dw->reverse_gc, GDK_SOLID);
+    if (dw->reverse_gc) gdk_gc_set_fill(dw->reverse_gc, GDK_SOLID);
+    gdk_window_raise(dw->win);
     dw_draw(dw);
 }
 
