@@ -2,7 +2,7 @@
 
 #include "config.h"
 #include "amitype.h"
-#include "hangulinput.h"
+#include "hinput.h"
 #include "handler.h"
 #include "ic.h"
 #include "dw.h"
@@ -162,11 +162,11 @@ ic_destroy(IC *ic)
 	cli_unset_status_ic(ic);
     }
     if (ic->top_win && ic == cli_get_edit_ic(ic)) {
-	draw_applet_image(-1);
+	draw_applet_image(AMI_PIXMAP_NONE);
 	cli_unset_edit_ic(ic);
     }
 
-    if (ic->top_win && ic->client_win != ic->top_win && !unique_han) {
+    if (ic->top_win && ic->client_win != ic->top_win && !unique_han && ic->client_win != GDK_ROOT_PARENT()) {
 	GdkWindowPrivate *private = (GdkWindowPrivate *)ic->top_win;
         gdk_window_unref(ic->top_win);
 	if (private->ref_count == 1) gdk_window_destroy_notify(ic->top_win);
@@ -190,7 +190,7 @@ ic_destroy(IC *ic)
     }
     ic->focus_win = NULL;
 	
-    if (ic->client_win) {
+    if (ic->client_win && ic->client_win != GDK_ROOT_PARENT()) {
 	GdkWindowPrivate *private = (GdkWindowPrivate *)ic->client_win;
 	gdk_window_unref(ic->client_win);
 	//if (private->ref_count == 1) gdk_window_destroy_notify(ic->client_win);
@@ -234,8 +234,10 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
 	//g_print("%s: ic_attr = %s; request\n", __FUNCTION__,ic_attr->name);
 	if (!strcmp(XNClientWindow, ic_attr->name)) {
 	  Window win = *(Window*)ic_attr->value;
+	  if (debug) g_print("client win set: xwin=%#lx ic = %d\n", win, ic->id);
 	  if (ic->client_win) {
 	    g_warning("client win is specified repeatedly");
+	    XFree(ic_attr->value);
 	    continue;
 	  }
 	  ic->client_win = cli_win_new(win, ic);
@@ -264,26 +266,33 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
 	  win = *(Window*)ic_attr->value;
 	  fwin = gdk_window_lookup(win);
 
-	  if (fwin && (fwin == ic->client_win || fwin == ic->focus_win)) continue;
+	  if (fwin && (/*fwin == ic->client_win ||*/ fwin == ic->focus_win)) {
+	     XFree(ic_attr->value);
+	     continue;
+	  }
 	  if (ic->focus_win) gdk_window_unref(ic->focus_win);
-	  if (fwin) {
-	    if (!check_focus_win(win)) {
+
+	  if (fwin == NULL) fwin = gdk_window_foreign_new(win);
+
+	  if (ic->edit_win && (ic->input_style & XIMPreeditPosition)) {
+	     dw_set_parent(ic->edit_win, fwin);
+	     if(debug) g_print("new edit win of ic(%d): %p %#lx\n", ic->id, ic->edit_win, GDK_WINDOW_XWINDOW(ic->edit_win));
+	  }
+
+	    if (check_focus_win(win)) {
 		GdkWindowPrivate *private = (GdkWindowPrivate *)fwin;
 		g_warning("focus win (%p) was already registered: ref=%d", fwin,private->ref_count);
 	    }
+	    gdk_window_ref(fwin);
 	    ic->focus_win = fwin;
-	  } else
-	      ic->focus_win = gdk_window_foreign_new(win);
 	  if (debug) g_print("focus_win = %p; ic=%d\n", ic->focus_win, ic->id);
-	  if (ic->edit_win && (ic->input_style & XIMPreeditPosition)) {
-	     dw_set_parent(ic->edit_win, ic->focus_win);
-	     if(debug) g_print("new edit win of ic(%d): %p %#lx\n", ic->id, ic->edit_win, GDK_WINDOW_XWINDOW(ic->edit_win));
-	  }
+	  XFree(ic_attr->value);
+	} else if (strcmp(XNInputStyle, ic_attr->name) != 0) {
+	  g_warning("%s: fail: %s", __FUNCTION__, ic_attr->name);
 	  XFree(ic_attr->value);
 	}
     }
 
-
     for (i = 0; i < (int)call_data->preedit_attr_num; i++, pre_attr++) {
 	if (ic->edit_win == NULL) {
 	    /* bogus */
@@ -322,12 +331,10 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
     for (i = 0; i < (int)call_data->status_attr_num; i++, sts_attr++) {
 	if (ic->status_win == NULL) {
 	    /* bogus */
-	    XFree(pre_attr->value);
+	    XFree(sts_attr->value);
 	    continue;
 	}
-	//g_print("%s: sts_attr = %s; request\n", __FUNCTION__,sts_attr->name);
 	if (!strcmp(XNArea, sts_attr->name)) {
-	  //g_print("set_area: ic=%d status\n", ic->id);
 	  dw_set_area(ic->status_win, (XRectangle*)sts_attr->value);
 	} else if (!strcmp(XNAreaNeeded, sts_attr->name)) {
 	  dw_set_area_needed(ic->status_win, (XRectangle*)sts_attr->value);
@@ -342,7 +349,7 @@ ic_store(IC *ic, IMChangeICStruct *call_data)
 	} else if (!strcmp(XNBackgroundPixmap, sts_attr->name))
 	  dw_set_bg_pixmap(ic->status_win, (Pixmap*)sts_attr->value);
 	else if (!strcmp(XNFontSet, sts_attr->name)) {
-	    dw_set_font(ic->status_win, sts_attr->value);
+	  dw_set_font(ic->status_win, sts_attr->value);
 	} else if (!strcmp(XNLineSpace, sts_attr->name))
 	  ic->status_win->linespace= *(CARD32*)sts_attr->value;
 	else if (!strcmp(XNCursor, sts_attr->name))
@@ -441,8 +448,12 @@ ic_get(IMChangeICStruct *call_data)
 	    *(CARD32*)ic_attr->value = KeyPressMask|KeyReleaseMask;
 	    //*(CARD32*)ic_attr->value = NoEventMask;
 	    ic_attr->value_length = sizeof(CARD32);
+	} else if (!strcmp(XNPreeditState, ic_attr->name)) {
+	    ic_attr->value = (void *)malloc(sizeof(CARD32));
+	    *(CARD32*)ic_attr->value = ic->composing_hangul;
+	    ic_attr->value_length = sizeof(CARD32);
 	} else {
-	    //g_print("%s: attr = %s; fail\n", __FUNCTION__,ic_attr->name);
+	    g_print("%s: attr = %s; fail\n", __FUNCTION__,ic_attr->name);
 	}
     }
 
